#!/usr/local/bin/perl -w
#
# Format:  add_to_database.cgi saved_msg_file
#
# This program takes its input from saved email message(s) that
# was sent by the calendar robot.  The format of how the msg(s)
# are saved should not matter since there are START and END tags
# in the msg themselves.  The number of msgs should not matter
# either; just put them all in one file.
#
# The database is saved in a file called 'calendar.db'.  Entries
# are all in time order so that the script making the calendar runs
# faster.  But THIS script is therefore a tad slow, needing to open,
# read, and close the DB for every entry.
#
# 
# check command line for argument
#
if ($ARGV[0] eq "") {
  die "Give MSG Filename for argument\n";
} else {
 open (MSGF, $ARGV[0]) || die "Can't open $ARGV[0]: $!\n";
}
#
# some array and hash setups
#
%monnames = qw(
January 00
February 01
March 02
April 03
May 04
June 05
July 06
August 07
September 08
October 09
November 10
December 11
);
#
# read in saved_msgs_file, saving all info:
#
$number_new_entries = 0;
$in_msg = 0;
while (<MSGF>) {                   # while there are lines left to be read in
  if (/^--xx START xx--/) {        # we have an event in this file
    $number_new_entries++;         # step count of new events
    $in_msg = 1;                   # flag to say we're in the middle of a message
    $_ = <MSGF>;
  }
  if (/^--xx END xx--/) {  
    $in_msg = 0;
  }
  if ($in_msg) {
      if (/^  Year of event:/) {     
        $year[$number_new_entries] = substr($_, 26, 4);
      } elsif (/^  Month of event:/) {     
        $month = substr($_, 26, 10);
        chomp($month);
        $thismonth[$number_new_entries] = $monnames{$month};
      } elsif (/^  Day of event:/) {    
        $day[$number_new_entries] = substr($_, 26, 2);
      } elsif (/^  Time of event:/) {    
        $tme[$number_new_entries] = substr($_, 26, 7);
        $tme[$number_new_entries] = "" if (/00:99none/);
      } elsif (/^  Location of event:/) {    
        $location[$number_new_entries] = substr($_, 26, 10);
        chomp($location[$number_new_entries]);
      } elsif (/^  Contact Name:/) {     
        $contname[$number_new_entries] = substr($_, 26, 40);
        chomp($contname[$number_new_entries]);
      } elsif (/^  Contact Email Address:/) {   
        $contemail[$number_new_entries] = substr($_, 26, 40);
        chomp($contemail[$number_new_entries]);
      } elsif (/^  Contact phone number:/) {   
        $contphone[$number_new_entries] = substr($_, 26, 14);
        chomp($contphone[$number_new_entries]);
      } elsif (/^  Brief description:/) {     
        $title[$number_new_entries] = substr($_, 26, 15);
        chomp($title[$number_new_entries]);
      } elsif (/^  xx Detailed description xx:/) {           # NEXT line has event details
        $_ = <MSGF>;
        $details[$number_new_entries] = substr($_, 0, 999);  
        chomp($details[$number_new_entries]);
        if ($details[$number_new_entries] eq "") {
          $details[$number_new_entries] = "No Details";
        }
        chomp($details[$number_new_entries]);
      } elsif (/^  xx Webpage xx:/) {                        # NEXT line has event webpage
        $_ = <MSGF>;
        $webpage[$number_new_entries] = substr($_, 0, 999);
        chomp($webpage[$number_new_entries]);
        if ($webpage[$number_new_entries] eq "") {
          $webpage[$number_new_entries] = "No WebPage";
        }
        chomp($webpage[$number_new_entries]);
      } else {
        print $_;
        die "unknown entry inside a message\n";
      }
  }        # end of this line entry in this event msg
}        # end of all MSG data
close (MSGF);
#
# concatenate year, month, day, and time to get timestamp for each entry
# then combine timestamp with contname and title to get first line
# of DB entry for each MSG; combine location, contemail, and contphone
# for next line; both details and webpage get their own lines.
# NB:  will assume no : is in contname, contemail, or contphone.
#
for ($i = 1; $i <= $number_new_entries; $i++) {
  if ($tme[$i] eq "") {
    $dummy_str = "     ";
    $hr_str[$i] = 99;
  } else {
    $dummy_1 = substr($tme[$i], 0, 2); # hrs
    $dummy_2 = substr($tme[$i], 3, 2); # mins
    $dummy_3 = substr($tme[$i], 5, 1); # a or p
    $dummy_str = $dummy_1 . $dummy_2 . $dummy_3;
    $hr_str[$i] = $dummy_1;
    if ( ($dummy_3 =~ /p/) && ($hr_str[$i] != 12) ) {
      $hr_str[$i] = $hr_str[$i] + 12;
    }
    if ($dummy_2 ne "  ") {
      $hr_str[$i] = $hr_str[$i] + $dummy_2 / 60.;
    }
    if ($hr_str[$i] < 10) {
      $hr_str[$i] = "0" . $hr_str[$i];
    }
  }
#
# the above hr_str is needed to sort entries in a given day since
# 12:00pm is larger in ASCII than say 04:00pm.
#
  $timestamp[$i] = $year[$i].$thismonth[$i].$day[$i].$dummy_str;
  $sort_timestamp[$i] = $year[$i].$thismonth[$i].$day[$i].$hr_str[$i];
  $line_1[$i] = $timestamp[$i].":".$contname[$i].":".$title[$i]."\n";
  $line_2[$i] = $contemail[$i].":".$contphone[$i].":".$location[$i]."\n";
  $line_3[$i] = $details[$i]."\n";
  $line_4[$i] = $webpage[$i]."\n";
  $not_written_to_DB[$i] = 1;
}
close(CALDB);
#
# now read in the DB itself (assumes it is already sorted) from
# the file calendar.db.  The old calendar.db will be renamed to
# calendar.db.old.  i will be the number of old entries.
#
open (CALDB, "<calendar.db") || die "Can't open calendar.db: $!\n";
$i = 0;
while (<CALDB>) {
  $i++;
  $dbline_1[$i] = $_;
  $dbline_2[$i] = <CALDB>;
  $dbline_3[$i] = <CALDB>;
  $dbline_4[$i] = <CALDB>;
  $dbtme_yr[$i] = substr($dbline_1[$i],0,4);
  $dbtme_mo[$i] = substr($dbline_1[$i],4,2);
  $dbtme_dy[$i] = substr($dbline_1[$i],6,2);
  $dbtme_hr[$i] = substr($dbline_1[$i],8,2);
  $dbtme_mn[$i] = substr($dbline_1[$i],10,2);
  $dummy_str = substr($dbline_1[$i],12,1);
  if ( ($dummy_str =~ /p/) && ($dbtme_hr[$i] < 12 || $dbtme_hr[$i] > 12) ) {
    $dbtme_hr[$i] = $dbtme_hr[$i] + 12;
  }
  if ($dbtme_mn[$i] ne "  ") {
    $dbtme_hr[$i] = $dbtme_hr[$i] + $dbtme_mn[$i] / 60.;
  }
  if ($dbtme_hr[$i] ne "  " && $dbtme_hr[$i] < 10) {
    $dbtme_hr[$i] = "0" . $dbtme_hr[$i];
  }
  $dbsort_timestamp[$i] = $dbtme_yr[$i].$dbtme_mo[$i].$dbtme_dy[$i].$dbtme_hr[$i];
}
rename("calendar.db","calendar.db.old") || die "Can't rename the calendar DB: $!\n";
close (CALDB) || die "Can't close the old calendar.db: $!\n";
#
# now create the new calendar.db inserting the new events 
#
open (CALDB, ">calendar.db") || die "Can't create the new calendar DB: $!\n";
for ($j = 1; $j <= $i; $j++) {
  for ($k = 1; $k <=$number_new_entries; $k++) {
    if (($sort_timestamp[$k] le $dbsort_timestamp[$j]) && ($not_written_to_DB[$k])) {
      $not_written_to_DB[$k] = 0;
      print CALDB $line_1[$k],$line_2[$k],$line_3[$k],$line_4[$k];
    }
  }
  print CALDB $dbline_1[$j],$dbline_2[$j],$dbline_3[$j],$dbline_4[$j];
}
#
# check for unwritten events; event may be last in new DB
#
for ($j = 1; $j <= $number_new_entries; $j++) {
  if ($not_written_to_DB[$j]) {
    if ($line_1[$j] gt $dbline_1[$i]) {
      print CALDB $line_1[$j],$line_2[$j],$line_3[$j],$line_4[$j];
    } else {
      print "ERROR -- Event not written to DB:\n";
      print $line_1[$j],$line_2[$j],$line_3[$j],$line_4[$j];
    }
  } 
}
close (CALDB) || die "Can't close the new calendar.db: $!\n";
